import java.io.*;
import java.net.*;
import com.lcmsoft.tftp.*;

/**
 * Simple command line, TFTP client application. This example is the most simple TFTP 
 * client possible. It demonstrates how to read files from or write files
 * to a TFTP server using just the bare minimum code needed to perform these 
 * operations. For that reason, the example does not support TFTP options and
 * uses the defaults provided by the TftpRequest class.
 *
 * Notice that all error handling code was intentionally removed in order to keep the
 * example short, clean and easy to understand. That also helps to focus the code
 * that does the actual job. Consequently, any error will generate an exception,
 * reported to the console. A real application would include code to check error
 * conditions and handle exceptions.
 *
 * The example accepts a command line comprised of three or four arguments:
 *
 *	- the TFTP server address, specified as an IP address or host name
 *	- the operation to perform, either the literal string 'read' or 'write'
 *	- the source file name, including any path information
 *	- the destination file name, including any path information
 *
 * The destination file name can be omitted, in that case the same name provided for
 * the source file will also be used for the destination. Notice that the example
 * does not translate or interpret the file names in any way, so they will be used
 * exactly as provided, including any path information.
 */
class SimpleTftpClient {
  public static void main( String[] args ) 
  	throws Exception {
	  
    /* Prints a usage message if the command line is wrong. Notice that only the number
       of parameters are checked. The parameters themselves are not checked and will
       generate an exception if they are not valid. */
    if ( (args.length < 3) || (args.length > 4) ) {
      System.out.println( "Usage: SimpleTftpClient server op srcfile dstfile" );
      System.out.println();
      System.out.println( "  server - TFTP server address" );
      System.out.println( "  op - 'read' or 'write'" );
      System.out.println( "  srcfile - source file name" );
      System.out.println( "  dstfile - destination file name (optional)" );
      System.exit( 0 );
    }
    
    /* Extract the parameters from the command line, saving them on appropriate local
       variables. Notice that, regarding the operation parameter, anything different 
       from the string 'read' (case-insensitive) is considered to be 'write'. */
    InetAddress server = InetAddress.getByName(args[0]);
    int operation = args[1].equalsIgnoreCase("read")?TftpRequest.READ:TftpRequest.WRITE;
    String srcFile = args[2];
    String dstFile = args.length>3?args[3]:srcFile;
    
    /* Constructs a TftpRequest object and set its properties according the provided
       parameters. Notice that only the required properties required to carry on the
       operation are set, all others are left to their default values. That means,
       for example, that the request will use the standard TFTP port and will not use 
       TFTP options. */
    TftpRequest request = new TftpRequest();
    request.setHost( server );
    request.setOperation( operation );
    request.setFileName( operation==TftpRequest.READ?srcFile:dstFile );
    
    /* Save the current time to calculate the transfer time later. */ 
    long time = System.currentTimeMillis();

    InputStream in;
    OutputStream out;
    if ( operation == TftpRequest.READ ) {
      /* If the request is to read a file from the server, gets an input stream to
         receive the file and an output stream to write it to the disk. */
      System.out.println( "Reading file "+srcFile+" from "+server+" to "+dstFile );
      in = request.getInputStream();
      out = new BufferedOutputStream(new FileOutputStream(dstFile));
    } else {
      /* If the request is to write a file to the server, gets an input stream to
         read the file from the disk and an output stream to send the data. */
      System.out.println( "Writing file "+srcFile+" to "+dstFile+" on "+server );
      in = new BufferedInputStream(new FileInputStream(srcFile));
      out = request.getOutputStream();
    }

    /* Transfer all data from the input to the output stream, that is, from the
       server to the local disk or vice-versa, depending on the operation (see 
       above). */
    int b;
    while ( (b = in.read()) != -1 )
      out.write( b );
    in.close();
    out.close();
    
    /* Calculates and prints how much time it took to transfer the file. */
    System.out.println( "Transfer complete" );
    System.out.println( "Elapsed time: "+((System.currentTimeMillis()-time)/1000)+"s" );
  }
}